"""Cloudflare API via command line"""
from __future__ import absolute_import

import CloudFlare

def convert_zones_to_identifier(cf, zone_name):
    """zone names to numbers"""
    params = {'name':zone_name, 'per_page':1}
    try:
        zones = cf.zones.get(params=params)
    except CloudFlare.exceptions.CloudFlareAPIError as e:
        exit('cli4: %s - %d %s' % (zone_name, e, e))
    except Exception as e:
        exit('cli4: %s - %s' % (zone_name, e))

    if len(zones) == 1:
        return zones[0]['id']

    exit('cli4: %s - zone not found' % (zone_name))

def convert_dns_record_to_identifier(cf, zone_id, dns_name):
    """dns record names to numbers"""
    # this can return an array of results as there can be more than one DNS entry for a name.
    params = {'name':dns_name}
    try:
        dns_records = cf.zones.dns_records.get(zone_id, params=params)
    except CloudFlare.exceptions.CloudFlareAPIError as e:
        exit('cli4: %s - %d %s' % (dns_name, e, e))
    except Exception as e:
        exit('cli4: %s - %s' % (dns_name, e))

    r = []
    for dns_record in dns_records:
        if dns_name == dns_record['name']:
            r.append(dns_record['id'])
    if len(r) > 0:
        return r

    exit('cli4: %s - dns name not found' % (dns_name))

def convert_certificates_to_identifier(cf, certificate_name):
    """certificate names to numbers"""
    try:
        certificates = cf.certificates.get()
    except CloudFlare.exceptions.CloudFlareAPIError as e:
        exit('cli4: %s - %d %s' % (certificate_name, e, e))
    except Exception as e:
        exit('cli4: %s - %s' % (certificate_name, e))

    for certificate in certificates:
        if certificate_name in certificate['hostnames']:
            return certificate['id']

    exit('cli4: %s - no zone certificates found' % (certificate_name))

def convert_organizations_to_identifier(cf, organization_name):
    """organizations names to numbers"""
    try:
        organizations = cf.user.organizations.get()
    except CloudFlare.exceptions.CloudFlareAPIError as e:
        exit('cli4: %s - %d %s' % (organization_name, e, e))
    except Exception as e:
        exit('cli4: %s - %s' % (organization_name, e))

    for organization in organizations:
        if organization_name == organization['name']:
            return organization['id']

    exit('cli4: %s - no organizations found' % (organization_name))

def convert_invites_to_identifier(cf, invite_name):
    """invite names to numbers"""
    try:
        invites = cf.user.invites.get()
    except CloudFlare.exceptions.CloudFlareAPIError as e:
        exit('cli4: %s - %d %s' % (invite_name, e, e))
    except Exception as e:
        exit('cli4: %s - %s' % (invite_name, e))

    for invite in invites:
        if invite_name == invite['organization_name']:
            return invite['id']

    exit('cli4: %s - no invites found' % (invite_name))

def convert_virtual_dns_to_identifier(cf, virtual_dns_name):
    """virtual dns names to numbers"""
    try:
        virtual_dnss = cf.user.virtual_dns.get()
    except CloudFlare.exceptions.CloudFlareAPIError as e:
        exit('cli4: %s - %d %s\n' % (virtual_dns_name, e, e))
    except Exception as e:
        exit('cli4: %s - %s\n' % (virtual_dns_name, e))

    for virtual_dns in virtual_dnss:
        if virtual_dns_name == virtual_dns['name']:
            return virtual_dns['id']

    exit('cli4: %s - no virtual_dns found' % (virtual_dns_name))

def convert_load_balancers_pool_to_identifier(cf, pool_name):
    """load balancer pool names to numbers"""
    try:
        pools = cf.user.load_balancers.pools.get()
    except CloudFlare.exceptions.CloudFlareAPIError as e:
        exit('cli4: %s - %d %s' % (pool_name, e, e))
    except Exception as e:
        exit('cli4: %s - %s' % (pool_name, e))

    for p in pools:
        if pool_name == p['description']:
            return p['id']

    exit('cli4: %s - no pools found' % (pool_name))




""" API extras for Cloudflare API"""

import re

def api_extras(self, extras=None):
    """ API extras for Cloudflare API"""

    for extra in extras:
        if extra == '':
            continue
        extra = re.sub(r"^.*/client/v4/", '/', extra)
        extra = re.sub(r"^.*/v4/", '/', extra)
        extra = re.sub(r"^/", '', extra)

        # build parts of the extra command
        parts = []
        nn = 0
        for element in extra.split('/'):
            if element[0] == ':':
                nn += 1
                continue
            try:
                parts[nn]
            except IndexError:
                parts.append([])
            parts[nn].append(element)

        # insert extra command into class
        element_path = []
        current = self
        for element in parts[0]:
            element_path.append(element)
            try:
                m = getattr(current, element)
                # exists - but still add it there's a second part
                if element == parts[0][-1] and len(parts) > 1:
                    api_call_part1 = '/'.join(element_path)
                    api_call_part2 = '/'.join(parts[1])
                    setattr(m, parts[1][0],
                            self._add_with_auth(self._base, api_call_part1, api_call_part2))
                current = m
                continue
            except:
                pass
            # does not exist
            if element == parts[0][-1] and len(parts) > 1:
                # last element
                api_call_part1 = '/'.join(element_path)
                api_call_part2 = '/'.join(parts[1])
                setattr(current, element,
                        self._add_with_auth(self._base, api_call_part1, api_call_part2))
            else:
                api_call_part1 = '/'.join(element_path)
                setattr(current, element,
                        self._add_with_auth(self._base, api_call_part1))
            current = getattr(current, element)




""" reading the config file for Cloudflare API"""

import os
import re
try:
    import ConfigParser  # py2
except ImportError:
    import configparser as ConfigParser  # py3

def read_configs():
    """ reading the config file for Cloudflare API"""

    # envioronment variables override config files
    email = os.getenv('CF_API_EMAIL')
    token = os.getenv('CF_API_KEY')
    certtoken = os.getenv('CF_API_CERTKEY')
    extras = os.getenv('CF_API_EXTRAS')

    # grab values from config files
    config = ConfigParser.RawConfigParser()
    config.read([
        '.cloudflare.cfg',
        os.path.expanduser('~/.cloudflare.cfg'),
        os.path.expanduser('~/.cloudflare/cloudflare.cfg')
    ])

    if email is None:
        try:
            email = re.sub(r"\s+", '', config.get('CloudFlare', 'email'))
        except (ConfigParser.NoOptionError, ConfigParser.NoSectionError):
            email = None

    if token is None:
        try:
            token = re.sub(r"\s+", '', config.get('CloudFlare', 'token'))
        except (ConfigParser.NoOptionError, ConfigParser.NoSectionError):
            token = None

    if certtoken is None:
        try:
            certtoken = re.sub(r"\s+", '', config.get('CloudFlare', 'certtoken'))
        except (ConfigParser.NoOptionError, ConfigParser.NoSectionError):
            certtoken = None

    if extras is None:
        try:
            extras = re.sub(r"\s+", ' ', config.get('CloudFlare', 'extras'))
        except (ConfigParser.NoOptionError, ConfigParser.NoSectionError):
            extras = None

        if extras:
            extras = extras.split(' ')

    return [email, token, certtoken, extras]




#!/usr/bin/env python
"""Cloudflare API code - example"""

from __future__ import print_function

import os
import sys
import json

sys.path.insert(0, os.path.abspath('..'))
import CloudFlare

def main():
    """Cloudflare API code - example"""

    cf = CloudFlare.CloudFlare()

    print('USER:')
    # grab the user info
    try:
        user = cf.user.get()
    except CloudFlare.exceptions.CloudFlareAPIError as e:
        exit('/user.get %d %s - api call failed' % (e, e))
    except Exception as e:
        exit('/user.get - %s - api call failed' % (e))
    for k in sorted(user.keys()):
        if isinstance(user[k], list):
            if isinstance(user[k][0], dict):
                print('\t%-40s =' % (k))
                for l in user[k]:
                    for j in sorted(l.keys()):
                        if isinstance(l[j], list):
                            print('\t%-40s   %s = [ %s ]' % ('', j, ', '.join(l[j])))
                        else:
                            print('\t%-40s   %s = %s' % ('', j, l[j]))
            else:
                print('\t%-40s = [ %s ]' % (k, ', '.join(user[k])))
        elif isinstance(user[k], dict):
            print('\t%-40s =' % (k))
            for j in sorted(user[k].keys()):
                print('\t%-40s   %s = %s' % ('', j, user[k][j]))
        else:
            print('\t%-40s = %s' % (k, user[k]))
    print('')

    print('ORGANIZATIONS:')
    # grab the user organizations info
    try:
        organizations = cf.user.organizations.get()
    except CloudFlare.exceptions.CloudFlareAPIError as e:
        exit('/user.organizations.get %d %s - api call failed' % (e, e))
    if len(organizations) == 0:
        print('\tNo organization')
    for organization in organizations:
        organization_name = organization['name']
        organization_id = organization['id']
        organization_status = organization['status']
        print('\t%-40s %-10s %s' % (organization_id, organization_status, organization_name))
    print('')

    print('INVITES:')
    # grab the user invites info
    try:
        invites = cf.user.invites.get()
    except CloudFlare.exceptions.CloudFlareAPIError as e:
        exit('/user.invites.get %d %s - api call failed' % (e, e))
    if len(invites) == 0:
        print('\tNo user invites')
    for invite in invites:
        invited_member_id = invite['invited_member_id']
        invited_member_email = invite['invited_member_email']
        organization_id = invite['organization_id']
        organization_name = invite['organization_name']
        invited_by = invite['invited_by']
        invited_on = invite['invited_on']
        expires_on = invite['expires_on']
        status = invite['status']
        print('\t %s %s %s %s %s %s %s %s' % (
            organization_id,
            status,
            invited_member_id,
            invited_member_email,
            organization_name,
            invited_by,
            invited_on,
            expires_on
        ))
    print('')

    print('BILLING:')
    # grab the user billing profile info
    try:
        profile = cf.user.billing.profile.get()
    except CloudFlare.exceptions.CloudFlareAPIError as e:
        exit('/user.billing.profile.get %d %s - api call failed' % (e, e))
    profile_id = profile['id']
    profile_first = profile['first_name']
    profile_last = profile['last_name']
    profile_company = profile['company']
    if profile_company is None:
        profile_company = ''

    if profile['payment_email'] != '':
        payment_email = profile['payment_email']
        card_number = None
        card_expiry_year = None
        card_expiry_month = None
    else:
        payment_email = None
        card_number = profile['card_number']
        card_expiry_year = profile['card_expiry_year']
        card_expiry_month = profile['card_expiry_month']

    if payment_email is not None:
        print('\t %s %s %s %s PayPal: %s' % (
            profile_id,
            profile_first,
            profile_last,
            profile_company,
            payment_email
        ))
    else:
        if card_number is None:
            card_number = '---- ---- ----- ----'
        if card_expiry_year is not None and card_expiry_month is not None:
            card_expiry = card_expiry_month + '/' + card_expiry_year
        else:
            card_expiry = '--/--'
        print('\t %s %s %s %s CC: %s %s' % (
            profile_id,
            profile_first,
            profile_last,
            profile_company,
            card_number,
            card_expiry
        ))

    print('')

    print('BILLING HISTORY:')
    # grab the user billing history info
    try:
        history = cf.user.billing.history.get()
    except CloudFlare.exceptions.CloudFlareAPIError as e:
        exit('/user.billing.history.get %d %s - api call failed' % (e, e))
    if len(history) == 0:
        print('\tNo billing history')
    for h in sorted(history, key=lambda v: v['occurred_at']):
        history_id = h['id']
        history_type = h['type']
        history_action = h['action']
        history_occurred_at = h['occurred_at']
        history_amount = h['amount']
        history_currency = h['currency']
        history_description = h['description']
        print('\t %s %s %s %s %s %s %s' % (
            history_id,
            history_type,
            history_action,
            history_occurred_at,
            history_amount,
            history_currency,
            history_description
        ))

    print('')

    exit(0)

if __name__ == '__main__':
    main()




#!/usr/bin/env python
"""Cloudflare API code - example"""

from __future__ import print_function

import os
import sys

sys.path.insert(0, os.path.abspath('..'))
import CloudFlare

def main():
    """Cloudflare API code - example"""

    # Grab the first argument, if there is one
    try:
        zone_name = sys.argv[1]
        params = {'name':zone_name, 'per_page':1}
    except IndexError:
        params = {'per_page':1}

    cf = CloudFlare.CloudFlare()

    # grab the zone identifier
    try:
        zones = cf.zones.get(params=params)
    except CloudFlare.exceptions.CloudFlareAPIError as e:
        exit('/zones.get %d %s - api call failed' % (e, e))
    except Exception as e:
        exit('/zones.get - %s - api call failed' % (e))

    # there should only be one zone
    for zone in sorted(zones, key=lambda v: v['name']):
        zone_name = zone['name']
        zone_id = zone['id']
        try:
            settings = cf.zones.settings.get(zone_id)
        except CloudFlare.exceptions.CloudFlareAPIError as e:
            exit('/zones.settings.get %d %s - api call failed' % (e, e))

        print(zone_id, zone_name)
        for setting in sorted(settings, key=lambda v: v['id']):
            r_name = setting['id']
            r_value = setting['value']
            r_editable = setting['editable']
            try:
                k = sorted(r_value.keys())
                print('\t%-30s %10s = %s' % (r_name, '(editable)' if r_editable else '', '{'))
                for k in sorted(r_value.keys()):
                    print('\t%-30s %10s    %s = %s' % ('', '', r_name+'/'+k, r_value[k]))
                print('\t%-30s %10s = %s' % ('', '', '}'))
            except:
                print('\t%-30s %10s = %s' % (r_name, '(editable)' if r_editable else '', r_value))

        print('')

    exit(0)

if __name__ == '__main__':
    main()




""" API core commands for Cloudflare API"""

def api_v4(self):
    """ API core commands for Cloudflare API"""

    # The API commands for /user/
    user(self)
    user_audit_logs(self)
    user_load_balancers(self)
    user_load_balancing_analytics(self)
    user_virtual_dns(self)
    user_workers(self)

    # The API commands for /zones/
    zones(self)
    zones_amp(self)
    zones_analytics(self)
    zones_argo(self)
    zones_dns_analytics(self)
    zones_dnssec(self)
    zones_firewall(self)
    zones_load_balancers(self)
    zones_logs(self)
    zones_rate_limits(self)
    zones_settings(self)
    zones_ssl(self)
    zones_workers(self)

    # The API commands for /railguns/
    railguns(self)

    # The API commands for /organizations/
    organizations(self)
    organizations_audit_logs(self)
    organizations_virtual_dns(self)
    organizations_workers(self)

    # The API commands for /certificates/
    certificates(self)

    # The API commands for /ips/
    ips(self)

    # The API commands for /account/
    account(self)
    account_load_balancing_analytics(self)

def user(self):
    """ API core commands for Cloudflare API"""

    base = self._base
    setattr(self, "user",
            self._add_with_auth(base, "user"))
    branch = self.user
    setattr(branch, "billing",
            self._add_unused(base, "user/billing"))
    branch = self.user.billing
    setattr(branch, "history",
            self._add_with_auth(base, "user/billing/history"))
    setattr(branch, "profile",
            self._add_with_auth(base, "user/billing/profile"))
    setattr(branch, "subscriptions",
            self._add_unused(base, "user/billing/subscriptions"))
    branch = self.user.billing.subscriptions
    setattr(branch, "apps",
            self._add_with_auth(base, "user/billing/subscriptions/apps"))
    setattr(branch, "zones",
            self._add_with_auth(base, "user/billing/subscriptions/zones"))
    branch = self.user
    setattr(branch, "firewall",
            self._add_unused(base, "user/firewall"))
    branch = self.user.firewall
    setattr(branch, "access_rules",
            self._add_unused(base, "user/firewall/access_rules"))
    branch = self.user.firewall.access_rules
    setattr(branch, "rules",
            self._add_with_auth(base, "user/firewall/access_rules/rules"))
    branch = self.user
    setattr(branch, "organizations",
            self._add_with_auth(base, "user/organizations"))
    setattr(branch, "invites",
            self._add_with_auth(base, "user/invites"))
    setattr(branch, "subscriptions",
            self._add_with_auth(base, "user/subscriptions"))

def zones(self):
    """ API core commands for Cloudflare API"""

    base = self._base
    setattr(self, "zones",
            self._add_with_auth(base, "zones"))
    branch = self.zones
    setattr(branch, "activation_check",
            self._add_with_auth(base, "zones", "activation_check"))
    setattr(branch, "available_plans",
            self._add_with_auth(base, "zones", "available_plans"))
    setattr(branch, "available_rate_plans",
            self._add_with_auth(base, "zones", "available_rate_plans"))
    setattr(branch, "custom_certificates",
            self._add_with_auth(base, "zones", "custom_certificates"))
    branch = self.zones.custom_certificates
    setattr(branch, "prioritize",
            self._add_with_auth(base, "zones", "custom_certificates/prioritize"))
    branch = self.zones
    setattr(branch, "custom_pages",
            self._add_with_auth(base, "zones", "custom_pages"))
    setattr(branch, "dns_records",
            self._add_with_auth(base, "zones", "dns_records"))
    setattr(branch, "keyless_certificates",
            self._add_with_auth(base, "zones", "keyless_certificates"))
    setattr(branch, "pagerules",
            self._add_with_auth(base, "zones", "pagerules"))
    setattr(branch, "purge_cache",
            self._add_with_auth(base, "zones", "purge_cache"))
    setattr(branch, "railguns",
            self._add_with_auth(base, "zones", "railguns"))
    branch = self.zones.railguns
    setattr(branch, "diagnose",
            self._add_with_auth(base, "zones", "railguns", "diagnose"))
    branch = self.zones
    setattr(branch, "subscription",
            self._add_with_auth(base, "zones", "subscription"))
    setattr(branch, "subscriptions",
            self._add_with_auth(base, "zones", "subscriptions"))
    branch = self.zones.dns_records
    setattr(branch, "export",
            self._add_with_auth(base, "zones", "dns_records/export"))
    setattr(branch, "import",
            self._add_with_auth(base, "zones", "dns_records/import"))
    branch = self.zones
    setattr(branch, "custom_hostnames",
            self._add_with_auth(base, "zones", "custom_hostnames"))

def zones_settings(self):
    """ API core commands for Cloudflare API"""

    base = self._base
    branch = self.zones
    setattr(branch, "settings",
            self._add_with_auth(base, "zones", "settings"))
    branch = self.zones.settings
    setattr(branch, "advanced_ddos",
            self._add_with_auth(base, "zones", "settings/advanced_ddos"))
    setattr(branch, "always_online",
            self._add_with_auth(base, "zones", "settings/always_online"))
    setattr(branch, "always_use_https",
            self._add_with_auth(base, "zones", "settings/always_use_https"))
    setattr(branch, "browser_cache_ttl",
            self._add_with_auth(base, "zones", "settings/browser_cache_ttl"))
    setattr(branch, "browser_check",
            self._add_with_auth(base, "zones", "settings/browser_check"))
    setattr(branch, "cache_level",
            self._add_with_auth(base, "zones", "settings/cache_level"))
    setattr(branch, "challenge_ttl",
            self._add_with_auth(base, "zones", "settings/challenge_ttl"))
    setattr(branch, "development_mode",
            self._add_with_auth(base, "zones", "settings/development_mode"))
    setattr(branch, "email_obfuscation",
            self._add_with_auth(base, "zones", "settings/email_obfuscation"))
    setattr(branch, "hotlink_protection",
            self._add_with_auth(base, "zones", "settings/hotlink_protection"))
    setattr(branch, "ip_geolocation",
            self._add_with_auth(base, "zones", "settings/ip_geolocation"))
    setattr(branch, "ipv6",
            self._add_with_auth(base, "zones", "settings/ipv6"))
    setattr(branch, "minify",
            self._add_with_auth(base, "zones", "settings/minify"))
    setattr(branch, "mirage",
            self._add_with_auth(base, "zones", "settings/mirage"))
    setattr(branch, "mobile_redirect",
            self._add_with_auth(base, "zones", "settings/mobile_redirect"))
    setattr(branch, "origin_error_page_pass_thru",
            self._add_with_auth(base, "zones", "settings/origin_error_page_pass_thru"))
    setattr(branch, "polish",
            self._add_with_auth(base, "zones", "settings/polish"))
    setattr(branch, "prefetch_preload",
            self._add_with_auth(base, "zones", "settings/prefetch_preload"))
    setattr(branch, "response_buffering",
            self._add_with_auth(base, "zones", "settings/response_buffering"))
    setattr(branch, "rocket_loader",
            self._add_with_auth(base, "zones", "settings/rocket_loader"))
    setattr(branch, "security_header",
            self._add_with_auth(base, "zones", "settings/security_header"))
    setattr(branch, "security_level",
            self._add_with_auth(base, "zones", "settings/security_level"))
    setattr(branch, "server_side_exclude",
            self._add_with_auth(base, "zones", "settings/server_side_exclude"))
    setattr(branch, "sort_query_string_for_cache",
            self._add_with_auth(base, "zones", "settings/sort_query_string_for_cache"))
    setattr(branch, "ssl",
            self._add_with_auth(base, "zones", "settings/ssl"))
    setattr(branch, "tls_client_auth",
            self._add_with_auth(base, "zones", "settings/tls_client_auth"))
    setattr(branch, "true_client_ip_header",
            self._add_with_auth(base, "zones", "settings/true_client_ip_header"))
    setattr(branch, "tls_1_2_only",
            self._add_with_auth(base, "zones", "settings/tls_1_2_only"))
    setattr(branch, "tls_1_3",
            self._add_with_auth(base, "zones", "settings/tls_1_3"))
    # setattr(branch, "tlsadd_auth",
    #         self._add_with_auth(base, "zones", "settings/tlsadd_auth"))
    # setattr(branch, "trueadd_ip_header",
    #         self._add_with_auth(base, "zones", "settings/trueadd_ip_header"))
    setattr(branch, "websockets",
            self._add_with_auth(base, "zones", "settings/websockets"))
    setattr(branch, "waf",
            self._add_with_auth(base, "zones", "settings/waf"))
    setattr(branch, "webp",
            self._add_with_auth(base, "zones", "settings/webp"))
    setattr(branch, "http2",
            self._add_with_auth(base, "zones", "settings/http2"))
    setattr(branch, "pseudo_ipv4",
            self._add_with_auth(base, "zones", "settings/pseudo_ipv4"))
    setattr(branch, "opportunistic_encryption",
            self._add_with_auth(base, "zones", "settings/opportunistic_encryption"))
    setattr(branch, "automatic_https_rewrites",
            self._add_with_auth(base, "zones", "settings/automatic_https_rewrites"))
    setattr(branch, "brotli",
            self._add_with_auth(base, "zones", "settings/brotli"))
    setattr(branch, "privacy_pass",
            self._add_with_auth(base, "zones", "settings/privacy_pass"))

def zones_analytics(self):
    """ API core commands for Cloudflare API"""

    base = self._base
    branch = self.zones
    setattr(branch, "analytics",
            self._add_unused(base, "zones", "analytics"))
    branch = self.zones.analytics
    setattr(branch, "colos",
            self._add_with_auth(base, "zones", "analytics/colos"))
    setattr(branch, "dashboard",
            self._add_with_auth(base, "zones", "analytics/dashboard"))

def zones_firewall(self):
    """ API core commands for Cloudflare API"""

    base = self._base
    branch = self.zones
    setattr(branch, "firewall",
            self._add_unused(branch, "zones", "firewall"))
    branch = self.zones.firewall
    setattr(branch, "access_rules",
            self._add_unused(base, "zones", "firewall/access_rules"))
    setattr(branch, "waf",
            self._add_unused(base, "zones", "firewall/waf"))
    branch = self.zones.firewall.waf
    setattr(branch, "packages",
            self._add_with_auth(base, "zones", "firewall/waf/packages"))
    branch = self.zones.firewall.waf.packages
    setattr(branch, "groups",
            self._add_with_auth(base, "zones", "firewall/waf/packages", "groups"))
    setattr(branch, "rules",
            self._add_with_auth(base, "zones", "firewall/waf/packages", "rules"))
    branch = self.zones.firewall.access_rules
    setattr(branch, "rules",
            self._add_with_auth(base, "zones", "firewall/access_rules/rules"))
    branch = self.zones.firewall
    setattr(branch, "lockdowns",
            self._add_with_auth(base, "zones", "firewall/lockdowns"))
    setattr(branch, "ua_rules",
            self._add_with_auth(base, "zones", "firewall/ua_rules"))

def zones_rate_limits(self):
    """ API core commands for Cloudflare API"""

    base = self._base
    branch = self.zones
    setattr(branch, "rate_limits",
            self._add_with_auth(base, "zones", "rate_limits"))

def zones_dns_analytics(self):
    """ API core commands for Cloudflare API"""

    base = self._base
    branch = self.zones
    setattr(branch, "dns_analytics",
            self._add_unused(base, "zones", "dns_analytics"))
    branch = self.zones.dns_analytics
    setattr(branch, "report",
            self._add_with_auth(base, "zones", "dns_analytics/report"))
    branch = self.zones.dns_analytics.report
    setattr(branch, "bytime",
            self._add_with_auth(base, "zones", "dns_analytics/report/bytime"))

def zones_amp(self):
    """ API core commands for Cloudflare API"""

    base = self._base
    branch = self.zones
    setattr(branch, "amp",
            self._add_unused(base, "zones", "amp"))
    branch = self.zones.amp
    setattr(branch, "viewer",
            self._add_with_auth(base, "zones", "amp/viewer"))

def zones_logs(self):
    """ API core commands for Cloudflare API"""

    base = self._base
    branch = self.zones
    setattr(branch, "logs",
            self._add_unused(base, "zones", "logs"))
    branch = self.zones.logs
    setattr(branch, "received",
            self._add_with_auth_unwrapped(base, "zones", "logs/received"))
    branch = self.zones.logs.received
    setattr(branch, "fields",
            self._add_with_auth_unwrapped(base, "zones", "logs/received/fields"))

def railguns(self):
    """ API core commands for Cloudflare API"""

    base = self._base
    setattr(self, "railguns",
            self._add_with_auth(base, "railguns"))
    branch = self.railguns
    setattr(branch, "zones",
            self._add_with_auth(base, "railguns", "zones"))

def organizations(self):
    """ API core commands for Cloudflare API"""

    base = self._base
    setattr(self, "organizations",
            self._add_with_auth(base, "organizations"))
    branch = self.organizations
    setattr(branch, "members",
            self._add_with_auth(base, "organizations", "members"))
    setattr(branch, "invite",
            self._add_with_auth(base, "organizations", "invite"))
    setattr(branch, "invites",
            self._add_with_auth(base, "organizations", "invites"))
    setattr(branch, "railguns",
            self._add_with_auth(base, "organizations", "railguns"))
    branch = self.organizations.railguns
    setattr(branch, "zones",
            self._add_with_auth(base, "organizations", "railguns", "zones"))
    branch = self.organizations
    setattr(branch, "roles",
            self._add_with_auth(base, "organizations", "roles"))
    setattr(branch, "firewall",
            self._add_unused(base, "organizations", "firewall"))
    branch = self.organizations.firewall
    setattr(branch, "access_rules",
            self._add_unused(base, "organizations", "firewall/access_rules"))
    branch = self.organizations.firewall.access_rules
    setattr(branch, "rules",
            self._add_with_auth(base, "organizations", "firewall/access_rules/rules"))
    branch = self.organizations
    setattr(branch, "load_balancers",
            self._add_unused(base, "organizations", "load_balancers"))
    branch = self.organizations.load_balancers
    setattr(branch, "monitors",
            self._add_with_auth(base, "organizations", "load_balancers/monitors"))
    setattr(branch, "pools",
            self._add_with_auth(base, "organizations", "load_balancers/pools"))

def certificates(self):
    """ API core commands for Cloudflare API"""

    base = self._base
    setattr(self, "certificates",
            self._add_with_cert_auth(base, "certificates"))

def ips(self):
    """ API core commands for Cloudflare API"""

    base = self._base
    setattr(self, "ips",
            self._add_noauth(base, "ips"))

def zones_argo(self):
    """ API core commands for Cloudflare API"""

    base = self._base
    branch = self.zones
    setattr(branch, "argo",
            self._add_unused(base, "zones", "argo"))
    branch = self.zones.argo
    setattr(branch, "tiered_caching",
            self._add_with_auth(base, "zones", "argo/tiered_caching"))
    setattr(branch, "smart_routing",
            self._add_with_auth(base, "zones", "argo/smart_routing"))

def zones_dnssec(self):
    """ API core commands for Cloudflare API"""

    base = self._base
    branch = self.zones
    setattr(branch, "dnssec",
            self._add_with_auth(base, "zones", "dnssec"))

def zones_ssl(self):
    """ API core commands for Cloudflare API"""

    base = self._base
    branch = self.zones
    setattr(branch, "ssl",
            self._add_unused(base, "zones", "ssl"))
    branch = self.zones.ssl
    setattr(branch, "analyze",
            self._add_with_auth(base, "zones", "ssl/analyze"))
    setattr(branch, "certificate_packs",
            self._add_with_auth(base, "zones", "ssl/certificate_packs"))
    setattr(branch, "verification",
            self._add_with_auth(base, "zones", "ssl/verification"))
    setattr(branch, "universal",
            self._add_unused(base, "zones", "ssl/universal"))
    branch = self.zones.ssl.universal
    setattr(branch, "settings",
            self._add_with_auth(base, "zones", "ssl/universal/settings"))

def zones_workers(self):
    """ API core commands for Cloudflare API"""

    base = self._base
    branch = self.zones
    setattr(branch, "workers",
            self._add_unused(base, "zones", "workers"))
    branch = self.zones.workers
    setattr(branch, "filters",
            self._add_with_auth(base, "zones", "workers/filters"))
    setattr(branch, "routes",
            self._add_with_auth(base, "zones", "workers/routes"))
    setattr(branch, "script",
            self._add_with_auth(base, "zones", "workers/script"))

def zones_load_balancers(self):
    """ API core commands for Cloudflare API"""

    base = self._base
    branch = self.zones
    setattr(branch, "load_balancers",
            self._add_with_auth(base, "zones", "load_balancers"))

def user_load_balancers(self):
    """ API core commands for Cloudflare API"""

    base = self._base
    branch = self.user
    setattr(branch, "load_balancers",
            self._add_unused(base, "user/load_balancers"))
    branch = self.user.load_balancers
    setattr(branch, "monitors",
            self._add_with_auth(base, "user/load_balancers/monitors"))
    setattr(branch, "pools",
            self._add_with_auth(base, "user/load_balancers/pools"))

def user_virtual_dns(self):
    """ API core commands for Cloudflare API"""

    base = self._base
    branch = self.user
    setattr(branch, "virtual_dns",
            self._add_with_auth(base, "user/virtual_dns"))
    branch = self.user.virtual_dns
    setattr(branch, "dns_analytics",
            self._add_unused(base, "user/virtual_dns", "dns_analytics"))
    branch = self.user.virtual_dns.dns_analytics
    setattr(branch, "report",
            self._add_with_auth(base, "user/virtual_dns", "dns_analytics/report"))
    branch = self.user.virtual_dns.dns_analytics.report
    setattr(branch, "bytime",
            self._add_with_auth(base, "user/virtual_dns", "dns_analytics/report/bytime"))

def user_workers(self):
    """ API core commands for Cloudflare API"""

    base = self._base
    branch = self.user
    setattr(branch, "workers",
            self._add_unused(base, "user/workers"))
    branch = self.user.workers
    setattr(branch, "scripts",
            self._add_with_auth(base, "user/workers/scripts"))

def organizations_virtual_dns(self):
    """ API core commands for Cloudflare API"""

    base = self._base
    branch = self.organizations
    setattr(branch, "virtual_dns",
            self._add_with_auth(base, "organizations", "virtual_dns"))
    branch = self.organizations.virtual_dns
    setattr(branch, "dns_analytics",
            self._add_unused(base, "organizations", "virtual_dns", "dns_analytics"))
    branch = self.organizations.virtual_dns.dns_analytics
    setattr(branch, "report",
            self._add_with_auth(base, "organizations", "virtual_dns", "dns_analytics/report"))
    branch = self.organizations.virtual_dns.dns_analytics.report
    setattr(branch, "bytime",
            self._add_with_auth(base, "organizations", "virtual_dns", "dns_analytics/report/bytime"))

def user_audit_logs(self):
    """ API core commands for Cloudflare API"""

    base = self._base
    branch = self.user
    setattr(branch, "audit_logs",
            self._add_with_auth(base, "user/audit_logs"))

def user_load_balancing_analytics(self):
    """ API core commands for Cloudflare API"""

    base = self._base
    branch = self.user
    setattr(branch, "load_balancing_analytics",
            self._add_unused(base, "user/load_balancing_analytics"))
    branch = self.user.load_balancing_analytics
    setattr(branch, "events",
            self._add_with_auth(base, "user/load_balancing_analytics/events"))
    setattr(branch, "entities",
            self._add_with_auth(base, "user/load_balancing_analytics/entities"))

def organizations_audit_logs(self):
    """ API core commands for Cloudflare API"""

    base = self._base
    branch = self.organizations
    setattr(branch, "audit_logs",
            self._add_with_auth(base, "organizations", "audit_logs"))

def organizations_workers(self):
    """ API core commands for Cloudflare API"""

    base = self._base
    branch = self.organizations
    setattr(branch, "workers",
            self._add_unused(base, "organizations", "workers"))
    branch = self.organizations.workers
    setattr(branch, "scripts",
            self._add_with_auth(base, "organizations", "workers/scripts"))

def account(self):
    """ API core commands for Cloudflare API"""

    base = self._base
    setattr(self, "account",
            self._add_unused(base, "account"))

def account_load_balancing_analytics(self):
    """ API core commands for Cloudflare API"""

    base = self._base
    branch = self.account
    setattr(branch, "load_balancing_analytics",
            self._add_unused(base, "account", "load_balancing_analytics"))
    branch = self.account.load_balancing_analytics
    setattr(branch, "events",
            self._add_with_auth(base, "account", "load_balancing_analytics/events"))
    setattr(branch, "entities",
            self._add_with_auth(base, "account", "load_balancing_analytics/entities"))




#!/usr/bin/env python
"""Cloudflare API code - example"""

from __future__ import print_function

import os
import sys
import re

sys.path.insert(0, os.path.abspath('..'))
import CloudFlare

def main():
    """Cloudflare API code - example"""

    # Grab the first argument, if there is one
    try:
        zone_name = sys.argv[1]
        params = {'name':zone_name, 'per_page':1}
    except IndexError:
        params = {'per_page':50}

    cf = CloudFlare.CloudFlare()

    # grab the zone identifier
    try:
        zones = cf.zones.get(params=params)
    except CloudFlare.exceptions.CloudFlareAPIError as e:
        exit('/zones %d %s - api call failed' % (e, e))
    except Exception as e:
        exit('/zones.get - %s - api call failed' % (e))

    # there should only be one zone
    for zone in sorted(zones, key=lambda v: v['name']):
        zone_name = zone['name']
        zone_id = zone['id']
        if 'email' in zone['owner']:
            zone_owner = zone['owner']['email']
        else:
            zone_owner = '"' + zone['owner']['name'] + '"'
        zone_plan = zone['plan']['name']

        try:
            dns_records = cf.zones.dns_records.get(zone_id)
        except CloudFlare.exceptions.CloudFlareAPIError as e:
            exit('/zones/dns_records %d %s - api call failed' % (e, e))

        print(zone_id, zone_name, zone_owner, zone_plan)

        prog = re.compile('\.*'+zone_name+'$')
        dns_records = sorted(dns_records, key=lambda v: prog.sub('', v['name']) + '_' + v['type'])
        for dns_record in dns_records:
            r_name = dns_record['name']
            r_type = dns_record['type']
            r_value = dns_record['content']
            r_ttl = dns_record['ttl']
            r_id = dns_record['id']
            print('\t%s %60s %6d %-5s %s' % (r_id, r_name, r_ttl, r_type, r_value))

        print('')

    exit(0)

if __name__ == '__main__':
    main()




#!/usr/bin/env python
"""Cloudflare API code - setup.py file"""
import re
from setuptools import setup, find_packages

_version_re = re.compile(r"__version__\s=\s'(.*)'")


def main():
    """Cloudflare API code - setup.py file"""

    with open('README.rst') as read_me:
        long_description = read_me.read()

    with open('CloudFlare/__init__.py', 'r') as f:
        version = _version_re.search(f.read()).group(1)

    setup(
        name='cloudflare',
        version=version,
        description='Python wrapper for the Cloudflare v4 API',
        long_description=long_description,
        author='Martin J. Levy',
        author_email='martin@cloudflare.com',
        # maintainer='Martin J. Levy',
        # maintainer_email='martin@cloudflare.com',
        url='https://github.com/cloudflare/python-cloudflare',
        license='MIT',
        packages=['cli4', 'examples']+find_packages(),
        #package_dir={'CloudFlare': 'lib'}
        #package_dir={'CloudFlare/examples': 'examples'},
        #package_data={'cloudflare-examples': ["examples/*"]},
        include_package_data=True,
        #data_files = [('man/man1', ['cli4/cli4.man'])],
        install_requires=['requests', 'future', 'pyyaml'],
        keywords='cloudflare',
        entry_points={
            'console_scripts': [
                'cli4=cli4.__main__:main'
            ]
        },
        classifiers=[
            'Development Status :: 5 - Production/Stable',
            'Intended Audience :: Developers',
            'Topic :: Software Development :: Libraries :: Python Modules',
            'License :: OSI Approved :: MIT License',
            'Programming Language :: Python :: 2',
            'Programming Language :: Python :: 2.6',
            'Programming Language :: Python :: 2.7',
            'Programming Language :: Python :: 3',
            'Programming Language :: Python :: 3.2',
            'Programming Language :: Python :: 3.3',
            'Programming Language :: Python :: 3.4',
            'Programming Language :: Python :: 3.5',
            'Programming Language :: Python :: 3.6'
        ]
    )


if __name__ == '__main__':
    main()



""" Logging for Cloudflare API"""
import logging

# try:
#     import http.client as http_client
# except ImportError:
#     # Python 2
#     import httplib as http_client

DEBUG = 0
INFO = 1

class CFlogger(object):
    """ Logging for Cloudflare API"""

    def __init__(self, level):
        """ Logging for Cloudflare API"""
        self.logger_level = self._get_logging_level(level)
        #logging.basicConfig(level=self.logger_level)
        request_logger = logging.getLogger("requests.packages.urllib3")
        request_logger.setLevel(self.logger_level)
        request_logger.propagate = level

    def getLogger(self):
        """ Logging for Cloudflare API"""
        # create logger
        logger = logging.getLogger('Python Cloudflare API v4')
        logger.setLevel(self.logger_level)

        ch = logging.StreamHandler()
        ch.setLevel(self.logger_level)

        # create formatter
        formatter = logging.Formatter('%(asctime)s - %(name)s - %(levelname)s - %(message)s')

        # add formatter to ch
        ch.setFormatter(formatter)

        # add ch to logger
        logger.addHandler(ch)

        # http_client.HTTPConnection.debuglevel = 1

        return logger

    def _get_logging_level(self, level):
        """ Logging for Cloudflare API"""
        if level is True:
            return logging.DEBUG
        else:
            return logging.INFO



#!/usr/bin/env python
"""Cloudflare API code - example"""

from __future__ import print_function

import os
import sys
import json

sys.path.insert(0, os.path.abspath('..'))
import CloudFlare

def main():
    """Cloudflare API code - example"""

    # Grab the first argument, if there is one
    try:
        zone_name = sys.argv[1]
        params = {'name':zone_name, 'per_page':1}
    except IndexError:
        params = {'per_page':50}

    cf = CloudFlare.CloudFlare()

    # grab the zone identifier
    try:
        zones = cf.zones.get(params=params)
    except CloudFlare.exceptions.CloudFlareAPIError as e:
        exit('/zones %d %s - api call failed' % (e, e))
    except Exception as e:
        exit('/zones - %s - api call failed' % (e))

    # there should only be one zone
    for zone in sorted(zones, key=lambda v: v['name']):
        zone_name = zone['name']
        zone_id = zone['id']
        try:
            certificates = cf.zones.ssl.certificate_packs.get(zone_id)
        except CloudFlare.exceptions.CloudFlareAPIError as e:
            exit('/zones.ssl.certificate_packs %d %s - api call failed' % (e, e))

        for certificate in certificates:
            certificate_type = certificate['type']
            primary_certificate = certificate['primary_certificate']
            certificate_hosts = certificate['hosts']
            certificate_sig = certificate['certificates'][0]['signature']
            certificate_sig_count = len(certificate['certificates'])
            if certificate_sig_count > 1:
                c = certificate['certificates'][0]
                print('%-40s %-10s %-32s    %-15s [ %s ]' % (
                    zone_name,
                    certificate_type,
                    primary_certificate,
                    c['signature'],
                    ','.join(certificate_hosts)
                ))
                nn = 0
                for c in certificate['certificates']:
                    nn += 1
                    if nn == 1:
                        next
                    print('%-40s %-10s %-32s %2d:%-15s [ %s ]' % (
                        '',
                        '',
                        '',
                        nn,
                        c['signature'],
                        ''
                    ))
            else:
                for c in certificate['certificates']:
                    print('%-40s %-10s %-32s    %-15s [ %s ]' % (
                        zone_name,
                        certificate_type,
                        primary_certificate,
                        c['signature'],
                        ','.join(certificate_hosts)
                    ))

    exit(0)

if __name__ == '__main__':
    main()




#!/usr/bin/env python
"""Cloudflare API code - example"""

from __future__ import print_function

import os
import sys
import re
import json
import requests

sys.path.insert(0, os.path.abspath('..'))
import CloudFlare

def my_ip_address():
    """Cloudflare API code - example"""

    # This list is adjustable - plus some v6 enabled services are needed
    # url = 'http://myip.dnsomatic.com'
    # url = 'http://www.trackip.net/ip'
    # url = 'http://myexternalip.com/raw'
    url = 'https://api.ipify.org'
    try:
        ip_address = requests.get(url).text
    except:
        exit('%s: failed' % (url))
    if ip_address == '':
        exit('%s: failed' % (url))

    if ':' in ip_address:
        ip_address_type = 'AAAA'
    else:
        ip_address_type = 'A'

    return ip_address, ip_address_type

def do_dns_update(cf, zone_name, zone_id, dns_name, ip_address, ip_address_type):
    """Cloudflare API code - example"""

    try:
        params = {'name':dns_name, 'match':'all', 'type':ip_address_type}
        dns_records = cf.zones.dns_records.get(zone_id, params=params)
    except CloudFlare.exceptions.CloudFlareAPIError as e:
        exit('/zones/dns_records %s - %d %s - api call failed' % (dns_name, e, e))

    updated = False

    # update the record - unless it's already correct
    for dns_record in dns_records:
        old_ip_address = dns_record['content']
        old_ip_address_type = dns_record['type']

        if ip_address_type not in ['A', 'AAAA']:
            # we only deal with A / AAAA records
            continue

        if ip_address_type != old_ip_address_type:
            # only update the correct address type (A or AAAA)
            # we don't see this becuase of the search params above
            print('IGNORED: %s %s ; wrong address family' % (dns_name, old_ip_address))
            continue

        if ip_address == old_ip_address:
            print('UNCHANGED: %s %s' % (dns_name, ip_address))
            updated = True
            continue

        # Yes, we need to update this record - we know it's the same address type

        dns_record_id = dns_record['id']
        dns_record = {
            'name':dns_name,
            'type':ip_address_type,
            'content':ip_address
        }
        try:
            dns_record = cf.zones.dns_records.put(zone_id, dns_record_id, data=dns_record)
        except CloudFlare.exceptions.CloudFlareAPIError as e:
            exit('/zones.dns_records.put %s - %d %s - api call failed' % (dns_name, e, e))
        print('UPDATED: %s %s -> %s' % (dns_name, old_ip_address, ip_address))
        updated = True

    if updated:
        return

    # no exsiting dns record to update - so create dns record
    dns_record = {
        'name':dns_name,
        'type':ip_address_type,
        'content':ip_address
    }
    try:
        dns_record = cf.zones.dns_records.post(zone_id, data=dns_record)
    except CloudFlare.exceptions.CloudFlareAPIError as e:
        exit('/zones.dns_records.post %s - %d %s - api call failed' % (dns_name, e, e))
    print('CREATED: %s %s' % (dns_name, ip_address))

def main():
    """Cloudflare API code - example"""

    try:
        dns_name = sys.argv[1]
    except IndexError:
        exit('usage: example-update-dynamic-dns.py fqdn-hostname')

    host_name, zone_name = dns_name.split('.', 1)

    ip_address, ip_address_type = my_ip_address()

    print('MY IP: %s %s' % (dns_name, ip_address))

    cf = CloudFlare.CloudFlare()

    # grab the zone identifier
    try:
        params = {'name':zone_name}
        zones = cf.zones.get(params=params)
    except CloudFlare.exceptions.CloudFlareAPIError as e:
        exit('/zones %d %s - api call failed' % (e, e))
    except Exception as e:
        exit('/zones.get - %s - api call failed' % (e))

    if len(zones) == 0:
        exit('/zones.get - %s - zone not found' % (zone_name))

    if len(zones) != 1:
        exit('/zones.get - %s - api call returned %d items' % (zone_name, len(zones)))

    zone = zones[0]

    zone_name = zone['name']
    zone_id = zone['id']

    do_dns_update(cf, zone_name, zone_id, dns_name, ip_address, ip_address_type)
    exit(0)

if __name__ == '__main__':
    main()

